@@ -1,8 +1,8 @@
 #!/bin/bash
 
 # Change the list according to your local conda/virtualenv env.
-CONDA_ENVS="py27-np16 py27-np17 py33-np18 py34-np19 py35-np19 py36-np111"
-COMPRESS_METHODS="zlib gzip bz2 xz lzma"
+CONDA_ENVS="py27-np16 py27-np18 py34-np110 py35-np112 py36-np114"
+COMPRESS_METHODS="zlib gzip bz2 xz lzma lz4"
 
 for i in $CONDA_ENVS
 do
