@@ -6,27 +6,21 @@ Test the hashing module.
 # Copyright (c) 2009 Gael Varoquaux
 # License: BSD Style, 3 clauses.
 
-import nose
 import time
 import hashlib
-import tempfile
-import os
 import sys
 import gc
 import io
 import collections
+import itertools
 import pickle
 import random
-
-from nose.tools import assert_equal, assert_not_equal
+from decimal import Decimal
 
 from joblib.hashing import hash
 from joblib.func_inspect import filter_args
 from joblib.memory import Memory
-from joblib.testing import assert_raises_regex
-from joblib.test.test_memory import env as test_memory_env
-from joblib.test.test_memory import setup_module as test_memory_setup_func
-from joblib.test.test_memory import teardown_module as test_memory_teardown_func
+from joblib.testing import raises, skipif, fixture, parametrize
 from joblib.test.common import np, with_numpy
 from joblib.my_exceptions import TransportableException
 from joblib._compat import PY3_OR_LATER
@@ -39,11 +33,6 @@ except NameError:
     unicode = lambda s: s
 
 
-def assert_less(a, b):
-    if a > b:
-        raise AssertionError("%r is not lower than %r")
-
-
 ###############################################################################
 # Helper functions for the tests
 def time_func(func, *args):
@@ -76,8 +65,8 @@ class Klass(object):
 
 class KlassWithCachedMethod(object):
 
-    def __init__(self):
-        mem = Memory(cachedir=test_memory_env['dir'])
+    def __init__(self, cachedir):
+        mem = Memory(cachedir=cachedir)
         self.f = mem.cache(self.f)
 
     def f(self, x):
@@ -87,75 +76,81 @@ class KlassWithCachedMethod(object):
 ###############################################################################
 # Tests
 
-def test_trival_hash():
-    """ Smoke test hash on various types.
-    """
-    obj_list = [1, 2, 1., 2., 1 + 1j, 2. + 1j,
-                'a', 'b',
-                (1, ), (1, 1, ), [1, ], [1, 1, ],
-                {1: 1}, {1: 2}, {2: 1},
-                None,
-                gc.collect,
-                [1, ].append,
-                # Next 2 sets have unorderable elements in python 3.
-                set(('a', 1)),
-                set(('a', 1, ('a', 1))),
-                # Next 2 dicts have unorderable type of keys in python 3.
-                {'a': 1, 1: 2},
-                {'a': 1, 1: 2, 'd': {'a': 1}},
-                ]
-    for obj1 in obj_list:
-        for obj2 in obj_list:
-            # Check that 2 objects have the same hash only if they are
-            # the same.
-            yield nose.tools.assert_equal, hash(obj1) == hash(obj2), \
-                obj1 is obj2
+input_list = [1, 2, 1., 2., 1 + 1j, 2. + 1j,
+              'a', 'b',
+              (1,), (1, 1,), [1, ], [1, 1, ],
+              {1: 1}, {1: 2}, {2: 1},
+              None,
+              gc.collect,
+              [1, ].append,
+              # Next 2 sets have unorderable elements in python 3.
+              set(('a', 1)),
+              set(('a', 1, ('a', 1))),
+              # Next 2 dicts have unorderable type of keys in python 3.
+              {'a': 1, 1: 2},
+              {'a': 1, 1: 2, 'd': {'a': 1}}]
+
+
+@parametrize('obj1', input_list)
+@parametrize('obj2', input_list)
+def test_trivial_hash(obj1, obj2):
+    """Smoke test hash on various types."""
+    # Check that 2 objects have the same hash only if they are the same.
+    are_hashes_equal = hash(obj1) == hash(obj2)
+    are_objs_identical = obj1 is obj2
+    assert are_hashes_equal == are_objs_identical
 
 
 def test_hash_methods():
     # Check that hashing instance methods works
     a = io.StringIO(unicode('a'))
-    nose.tools.assert_equal(hash(a.flush), hash(a.flush))
+    assert hash(a.flush) == hash(a.flush)
     a1 = collections.deque(range(10))
     a2 = collections.deque(range(9))
-    nose.tools.assert_not_equal(hash(a1.extend), hash(a2.extend))
+    assert hash(a1.extend) != hash(a2.extend)
 
 
+@fixture(scope='function')
 @with_numpy
-def test_hash_numpy():
-    """ Test hashing with numpy arrays.
-    """
+def three_np_arrays():
     rnd = np.random.RandomState(0)
     arr1 = rnd.random_sample((10, 10))
     arr2 = arr1.copy()
     arr3 = arr2.copy()
     arr3[0] += 1
-    obj_list = (arr1, arr2, arr3)
-    for obj1 in obj_list:
-        for obj2 in obj_list:
-            yield nose.tools.assert_equal, hash(obj1) == hash(obj2), \
-                np.all(obj1 == obj2)
+    return arr1, arr2, arr3
 
-    d1 = {1: arr1, 2: arr1}
-    d2 = {1: arr2, 2: arr2}
-    yield nose.tools.assert_equal, hash(d1), hash(d2)
 
+def test_hash_numpy_arrays(three_np_arrays):
+    arr1, arr2, arr3 = three_np_arrays
+
+    for obj1, obj2 in itertools.product(three_np_arrays, repeat=2):
+        are_hashes_equal = hash(obj1) == hash(obj2)
+        are_arrays_equal = np.all(obj1 == obj2)
+        assert are_hashes_equal == are_arrays_equal
+
+    assert hash(arr1) != hash(arr1.T)
+
+
+def test_hash_numpy_dict_of_arrays(three_np_arrays):
+    arr1, arr2, arr3 = three_np_arrays
+
+    d1 = {1: arr1, 2: arr2}
+    d2 = {1: arr2, 2: arr1}
     d3 = {1: arr2, 2: arr3}
-    yield nose.tools.assert_not_equal, hash(d1), hash(d3)
 
-    yield nose.tools.assert_not_equal, hash(arr1), hash(arr1.T)
+    assert hash(d1) == hash(d2)
+    assert hash(d1) != hash(d3)
 
 
 @with_numpy
-def test_numpy_datetime_array():
+@parametrize('dtype', ['datetime64[s]', 'timedelta64[D]'])
+def test_numpy_datetime_array(dtype):
     # memoryview is not supported for some dtypes e.g. datetime64
     # see https://github.com/joblib/joblib/issues/188 for more details
-    dtypes = ['datetime64[s]', 'timedelta64[D]']
-
     a_hash = hash(np.arange(10))
-    arrays = (np.arange(0, 10, dtype=dtype) for dtype in dtypes)
-    for array in arrays:
-        nose.tools.assert_not_equal(hash(array), a_hash)
+    array = np.arange(0, 10, dtype=dtype)
+    assert hash(array) != a_hash
 
 
 @with_numpy
@@ -163,26 +158,23 @@ def test_hash_numpy_noncontiguous():
     a = np.asarray(np.arange(6000).reshape((1000, 2, 3)),
                    order='F')[:, :1, :]
     b = np.ascontiguousarray(a)
-    nose.tools.assert_not_equal(hash(a), hash(b))
+    assert hash(a) != hash(b)
 
     c = np.asfortranarray(a)
-    nose.tools.assert_not_equal(hash(a), hash(c))
+    assert hash(a) != hash(c)
 
 
 @with_numpy
-def test_hash_memmap():
-    """ Check that memmap and arrays hash identically if coerce_mmap is
-        True.
-    """
-    filename = tempfile.mktemp(prefix='joblib_test_hash_memmap_')
+@parametrize('coerce_mmap', [True, False])
+def test_hash_memmap(tmpdir, coerce_mmap):
+    """Check that memmap and arrays hash identically if coerce_mmap is True."""
+    filename = tmpdir.join('memmap_temp').strpath
     try:
         m = np.memmap(filename, shape=(10, 10), mode='w+')
         a = np.asarray(m)
-        for coerce_mmap in (False, True):
-            yield (nose.tools.assert_equal,
-                            hash(a, coerce_mmap=coerce_mmap)
-                                == hash(m, coerce_mmap=coerce_mmap),
-                            coerce_mmap)
+        are_hashes_equal = (hash(a, coerce_mmap=coerce_mmap) ==
+                            hash(m, coerce_mmap=coerce_mmap))
+        assert are_hashes_equal == coerce_mmap
     finally:
         if 'm' in locals():
             del m
@@ -190,15 +182,11 @@ def test_hash_memmap():
             # object is delete, and we don't run in a problem under
             # Windows with a file handle still open.
             gc.collect()
-            try:
-                os.unlink(filename)
-            except OSError as e:
-                # Under windows, some files don't get erased.
-                if not os.name == 'nt':
-                    raise e
 
 
 @with_numpy
+@skipif(sys.platform == 'win32', reason='This test is not stable under windows'
+                                        ' for some reason')
 def test_hash_numpy_performance():
     """ Check the performance of hashing numpy arrays:
 
@@ -216,10 +204,6 @@ def test_hash_numpy_performance():
         In [26]: %timeit hash(a)
         100 loops, best of 3: 20.8 ms per loop
     """
-    # This test is not stable under windows for some reason, skip it.
-    if sys.platform == 'win32':
-        raise nose.SkipTest()
-
     rnd = np.random.RandomState(0)
     a = rnd.random_sample(1000000)
     if hasattr(np, 'getbuffer'):
@@ -230,7 +214,7 @@ def test_hash_numpy_performance():
     md5_hash = lambda x: hashlib.md5(getbuffer(x)).hexdigest()
 
     relative_diff = relative_time(md5_hash, hash, a)
-    assert_less(relative_diff, 0.3)
+    assert relative_diff < 0.3
 
     # Check that hashing an tuple of 3 arrays takes approximately
     # 3 times as much as hashing one array
@@ -238,7 +222,7 @@ def test_hash_numpy_performance():
     time_hash = time_func(hash, (a, a, a))
     relative_diff = 0.5 * (abs(time_hash - time_hashlib)
                            / (time_hash + time_hashlib))
-    assert_less(relative_diff, 0.3)
+    assert relative_diff < 0.3
 
 
 def test_bound_methods_hash():
@@ -247,19 +231,18 @@ def test_bound_methods_hash():
     """
     a = Klass()
     b = Klass()
-    nose.tools.assert_equal(hash(filter_args(a.f, [], (1, ))),
-                            hash(filter_args(b.f, [], (1, ))))
+    assert (hash(filter_args(a.f, [], (1, ))) ==
+            hash(filter_args(b.f, [], (1, ))))
 
 
-@nose.tools.with_setup(test_memory_setup_func, test_memory_teardown_func)
-def test_bound_cached_methods_hash():
+def test_bound_cached_methods_hash(tmpdir):
     """ Make sure that calling the same _cached_ method on two different
     instances of the same class does resolve to the same hashes.
     """
-    a = KlassWithCachedMethod()
-    b = KlassWithCachedMethod()
-    nose.tools.assert_equal(hash(filter_args(a.f.func, [], (1, ))),
-                            hash(filter_args(b.f.func, [], (1, ))))
+    a = KlassWithCachedMethod(tmpdir.strpath)
+    b = KlassWithCachedMethod(tmpdir.strpath)
+    assert (hash(filter_args(a.f.func, [], (1, ))) ==
+            hash(filter_args(b.f.func, [], (1, ))))
 
 
 @with_numpy
@@ -269,8 +252,7 @@ def test_hash_object_dtype():
     a = np.array([np.arange(i) for i in range(6)], dtype=object)
     b = np.array([np.arange(i) for i in range(6)], dtype=object)
 
-    nose.tools.assert_equal(hash(a),
-                            hash(b))
+    assert hash(a) == hash(b)
 
 
 @with_numpy
@@ -279,14 +261,13 @@ def test_numpy_scalar():
     # strange pickling paths explored, that can give hash collisions
     a = np.float64(2.0)
     b = np.float64(3.0)
-    nose.tools.assert_not_equal(hash(a), hash(b))
+    assert hash(a) != hash(b)
 
 
-@nose.tools.with_setup(test_memory_setup_func, test_memory_teardown_func)
-def test_dict_hash():
+def test_dict_hash(tmpdir):
     # Check that dictionaries hash consistently, eventhough the ordering
     # of the keys is not garanteed
-    k = KlassWithCachedMethod()
+    k = KlassWithCachedMethod(tmpdir.strpath)
 
     d = {'#s12069__c_maps.nii.gz': [33],
          '#s12158__c_maps.nii.gz': [33],
@@ -305,15 +286,13 @@ def test_dict_hash():
     a = k.f(d)
     b = k.f(a)
 
-    nose.tools.assert_equal(hash(a),
-                            hash(b))
+    assert hash(a) == hash(b)
 
 
-@nose.tools.with_setup(test_memory_setup_func, test_memory_teardown_func)
-def test_set_hash():
-    # Check that sets hash consistently, eventhough their ordering
-    # is not garanteed
-    k = KlassWithCachedMethod()
+def test_set_hash(tmpdir):
+    # Check that sets hash consistently, even though their ordering
+    # is not guaranteed
+    k = KlassWithCachedMethod(tmpdir.strpath)
 
     s = set(['#s12069__c_maps.nii.gz',
              '#s12158__c_maps.nii.gz',
@@ -332,7 +311,14 @@ def test_set_hash():
     a = k.f(s)
     b = k.f(a)
 
-    nose.tools.assert_equal(hash(a), hash(b))
+    assert hash(a) == hash(b)
+
+
+def test_set_decimal_hash():
+    # Check that sets containing decimals hash consistently, even though
+    # ordering is not guaranteed
+    assert (hash(set([Decimal(0), Decimal('NaN')])) ==
+            hash(set([Decimal('NaN'), Decimal(0)])))
 
 
 def test_string():
@@ -342,7 +328,7 @@ def test_string():
     a = {string: 'bar'}
     b = {string: 'bar'}
     c = pickle.loads(pickle.dumps(b))
-    assert_equal(hash([a, b]), hash([a, c]))
+    assert hash([a, b]) == hash([a, c])
 
 
 @with_numpy
@@ -353,43 +339,37 @@ def test_dtype():
     a = np.dtype([('f1', np.uint), ('f2', np.int32)])
     b = a
     c = pickle.loads(pickle.dumps(a))
-    assert_equal(hash([a, c]), hash([a, b]))
+    assert hash([a, c]) == hash([a, b])
 
 
-def test_hashes_stay_the_same():
+@parametrize('to_hash,expected',
+             [('This is a string to hash',
+                 {'py2': '80436ada343b0d79a99bfd8883a96e45',
+                  'py3': '71b3f47df22cb19431d85d92d0b230b2'}),
+              (u"C'est l\xe9t\xe9",
+                 {'py2': '2ff3a25200eb6219f468de2640913c2d',
+                  'py3': '2d8d189e9b2b0b2e384d93c868c0e576'}),
+              ((123456, 54321, -98765),
+                 {'py2': '50d81c80af05061ac4dcdc2d5edee6d6',
+                  'py3': 'e205227dd82250871fa25aa0ec690aa3'}),
+              ([random.Random(42).random() for _ in range(5)],
+                 {'py2': '1a36a691b2e2ba3a9df72de3dccf17ea',
+                  'py3': 'a11ffad81f9682a7d901e6edc3d16c84'}),
+              ([3, 'abc', None, TransportableException('foo', ValueError)],
+                 {'py2': 'adb6ba84990ee5e462dc138383f11802',
+                  'py3': '994f663c64ba5e64b2a85ebe75287829'}),
+              ({'abcde': 123, 'sadfas': [-9999, 2, 3]},
+                 {'py2': 'fc9314a39ff75b829498380850447047',
+                  'py3': 'aeda150553d4bb5c69f0e69d51b0e2ef'})])
+def test_hashes_stay_the_same(to_hash, expected):
     # We want to make sure that hashes don't change with joblib
     # version. For end users, that would mean that they have to
     # regenerate their cache from scratch, which potentially means
     # lengthy recomputations.
-    rng = random.Random(42)
-    to_hash_list = ['This is a string to hash',
-                    u"C'est l\xe9t\xe9",
-                    (123456, 54321, -98765),
-                    [rng.random() for _ in range(5)],
-                    [3, 'abc', None,
-                     TransportableException('the message', ValueError)],
-                    {'abcde': 123, 'sadfas': [-9999, 2, 3]}]
-
-    # These expected results have been generated with joblib 0.9.2
-    expected_dict = {
-        'py2': ['80436ada343b0d79a99bfd8883a96e45',
-                '2ff3a25200eb6219f468de2640913c2d',
-                '50d81c80af05061ac4dcdc2d5edee6d6',
-                '536af09b66a087ed18b515acc17dc7fc',
-                '123ffc6f13480767167e171a8e1f6f4a',
-                'fc9314a39ff75b829498380850447047'],
-        'py3': ['71b3f47df22cb19431d85d92d0b230b2',
-                '2d8d189e9b2b0b2e384d93c868c0e576',
-                'e205227dd82250871fa25aa0ec690aa3',
-                '9e4e9bf9b91890c9734a6111a35e6633',
-                '6065a3c48e842ea5dee2cfd0d6820ad6',
-                'aeda150553d4bb5c69f0e69d51b0e2ef']}
+    # Expected results have been generated with joblib 0.9.2
 
     py_version_str = 'py3' if PY3_OR_LATER else 'py2'
-    expected_list = expected_dict[py_version_str]
-
-    for to_hash, expected in zip(to_hash_list, expected_list):
-        yield assert_equal, hash(to_hash), expected
+    assert hash(to_hash) == expected[py_version_str]
 
 
 @with_numpy
@@ -399,7 +379,7 @@ def test_hashes_are_different_between_c_and_fortran_contiguous_arrays():
     rng = np.random.RandomState(0)
     arr_c = rng.random_sample((10, 10))
     arr_f = np.asfortranarray(arr_c)
-    assert_not_equal(hash(arr_c), hash(arr_f))
+    assert hash(arr_c) != hash(arr_f)
 
 
 @with_numpy
@@ -409,7 +389,7 @@ def test_0d_array():
 
 @with_numpy
 def test_0d_and_1d_array_hashing_is_different():
-    assert_not_equal(hash(np.array(0)), hash(np.array([0])))
+    assert hash(np.array(0)) != hash(np.array([0]))
 
 
 @with_numpy
@@ -464,13 +444,13 @@ def test_hashes_stay_the_same_with_numpy_objects():
     expected_list = expected_dict[py_version_str]
 
     for to_hash, expected in zip(to_hash_list, expected_list):
-        yield assert_equal, hash(to_hash), expected
+        assert hash(to_hash) == expected
 
 
 def test_hashing_pickling_error():
     def non_picklable():
         return 42
 
-    assert_raises_regex(pickle.PicklingError,
-                        'PicklingError while hashing',
-                        hash, non_picklable)
+    with raises(pickle.PicklingError) as excinfo:
+        hash(non_picklable)
+    excinfo.match('PicklingError while hashing')
