@@ -57,6 +57,26 @@ def check_identity_lazy(func, accumulator, location):
             assert len(accumulator) == i + 1
 
 
+def corrupt_single_cache_item(memory):
+    single_cache_item, = memory.store_backend.get_items()
+    output_filename = os.path.join(single_cache_item.path, 'output.pkl')
+    with open(output_filename, 'w') as f:
+        f.write('garbage')
+
+
+def monkeypatch_cached_func_warn(func, monkeypatch_fixture):
+    # Need monkeypatch because pytest does not
+    # capture stdlib logging output (see
+    # https://github.com/pytest-dev/pytest/issues/2079)
+
+    recorded = []
+
+    def append_to_record(item):
+        recorded.append(item)
+    monkeypatch_fixture.setattr(func, 'warn', append_to_record)
+    return recorded
+
+
 ###############################################################################
 # Tests
 def test_memory_integration(tmpdir):
@@ -286,7 +306,7 @@ def test_memory_numpy(tmpdir, mmap_mode):
 
 
 @with_numpy
-def test_memory_numpy_check_mmap_mode(tmpdir):
+def test_memory_numpy_check_mmap_mode(tmpdir, monkeypatch):
     """Check that mmap_mode is respected even at the first call"""
 
     memory = Memory(location=tmpdir.strpath, mmap_mode='r', verbose=0)
@@ -306,6 +326,23 @@ def test_memory_numpy_check_mmap_mode(tmpdir):
     assert isinstance(b, np.memmap)
     assert b.mode == 'r'
 
+    # Corrupts the file,  Deleting b and c mmaps
+    # is necessary to be able edit the file
+    del b
+    del c
+    corrupt_single_cache_item(memory)
+
+    # Make sure that corrupting the file causes recomputation and that
+    # a warning is issued.
+    recorded_warnings = monkeypatch_cached_func_warn(twice, monkeypatch)
+    d = twice(a)
+    assert len(recorded_warnings) == 1
+    exception_msg = 'Exception while loading results'
+    assert exception_msg in recorded_warnings[0]
+    # Asserts that the recomputation returns a mmap
+    assert isinstance(d, np.memmap)
+    assert d.mode == 'r'
+
 
 def test_memory_exception(tmpdir):
     """ Smoketest the exception handling of Memory.
@@ -477,6 +514,42 @@ def test_call_and_shelve_argument_hash(tmpdir):
         in str(w[-1].message)
 
 
+def test_call_and_shelve_lazily_load_stored_result(tmpdir):
+    """Check call_and_shelve only load stored data if needed."""
+    test_access_time_file = tmpdir.join('test_access')
+    test_access_time_file.write('test_access')
+    test_access_time = os.stat(test_access_time_file.strpath).st_atime
+    # check file system access time stats resolution is lower than test wait
+    # timings.
+    time.sleep(0.5)
+    assert test_access_time_file.read() == 'test_access'
+
+    if test_access_time == os.stat(test_access_time_file.strpath).st_atime:
+        # Skip this test when access time cannot be retrieved with enough
+        # precision from the file system (e.g. NTFS on windows).
+        pytest.skip("filesystem does not support fine-grained access time "
+                    "attribute")
+
+    memory = Memory(location=tmpdir.strpath, verbose=0)
+    func = memory.cache(f)
+    func_id, argument_hash = func._get_output_identifiers(2)
+    result_path = os.path.join(memory.store_backend.location,
+                               func_id, argument_hash, 'output.pkl')
+    assert func(2) == 5
+    first_access_time = os.stat(result_path).st_atime
+    time.sleep(1)
+
+    # Should not access the stored data
+    result = func.call_and_shelve(2)
+    assert isinstance(result, MemorizedResult)
+    assert os.stat(result_path).st_atime == first_access_time
+    time.sleep(1)
+
+    # Read the stored data => last access time is greater than first_access
+    assert result.get() == 5
+    assert os.stat(result_path).st_atime > first_access_time
+
+
 def test_memorized_pickling(tmpdir):
     for func in (MemorizedFunc(f, tmpdir.strpath), NotMemorizedFunc(f)):
         filename = tmpdir.join('pickling_test.dat').strpath
@@ -836,8 +909,8 @@ def test_cached_function_race_condition_when_persisting_output_2(tmpdir,
     assert exception_msg not in stderr
 
 
-def test_memory_recomputes_after_an_error_why_loading_results(tmpdir,
-                                                              monkeypatch):
+def test_memory_recomputes_after_an_error_while_loading_results(
+        tmpdir, monkeypatch):
     memory = Memory(location=tmpdir.strpath)
 
     def func(arg):
@@ -856,21 +929,11 @@ def test_memory_recomputes_after_an_error_why_loading_results(tmpdir,
 
     # Corrupting output.pkl to make sure that an error happens when
     # loading the cached result
-    single_cache_item, = memory.store_backend.get_items()
-    output_filename = os.path.join(single_cache_item.path, 'output.pkl')
-    with open(output_filename, 'w') as f:
-        f.write('garbage')
-
-    recorded_warnings = []
-
-    def append_to_record(item):
-        recorded_warnings.append(item)
+    corrupt_single_cache_item(memory)
 
     # Make sure that corrupting the file causes recomputation and that
-    # a warning is issued. Need monkeypatch because pytest does not
-    # capture stdlib logging output (see
-    # https://github.com/pytest-dev/pytest/issues/2079)
-    monkeypatch.setattr(cached_func, 'warn', append_to_record)
+    # a warning is issued.
+    recorded_warnings = monkeypatch_cached_func_warn(cached_func, monkeypatch)
     recomputed_arg, recomputed_timestamp = cached_func(arg)
     assert len(recorded_warnings) == 1
     exception_msg = 'Exception while loading results'
@@ -878,6 +941,20 @@ def test_memory_recomputes_after_an_error_why_loading_results(tmpdir,
     assert recomputed_arg == arg
     assert recomputed_timestamp > timestamp
 
+    # Corrupting output.pkl to make sure that an error happens when
+    # loading the cached result
+    corrupt_single_cache_item(memory)
+    reference = cached_func.call_and_shelve(arg)
+    try:
+        reference.get()
+        raise AssertionError(
+            "It normally not possible to load a corrupted"
+            " MemorizedResult"
+        )
+    except KeyError as e:
+        message = "is corrupted"
+        assert message in str(e.args)
+
 
 def test_deprecated_cachedir_behaviour(tmpdir):
     # verify the right deprecation warnings are raised when using cachedir
