@@ -305,9 +305,9 @@ def test_parallel_pickling():
     """
     class UnpicklableObject(object):
         def __reduce__(self):
-            raise RuntimeError()
+            raise RuntimeError('123')
 
-    with raises(PicklingError):
+    with raises(PicklingError, match=r"the task to send"):
         Parallel(n_jobs=2)(delayed(id)(UnpicklableObject()) for _ in range(10))
 
 
@@ -494,8 +494,18 @@ def test_exception_dispatch():
             delayed(exception_raiser)(i) for i in range(30))
 
 
+def nested_function_inner(i):
+    Parallel(n_jobs=2)(
+        delayed(exception_raiser)(j) for j in range(30))
+
+
+def nested_function_outer(i):
+    Parallel(n_jobs=2)(
+        delayed(nested_function_inner)(j) for j in range(30))
+
+
 @with_multiprocessing
-@parametrize('backend', ['loky', 'multiprocessing', 'threading'])
+@parametrize('backend', PARALLEL_BACKENDS)
 def test_nested_exception_dispatch(backend):
     """Ensure errors for nested joblib cases gets propagated
 
@@ -509,14 +519,6 @@ def test_nested_exception_dispatch(backend):
         raise SkipTest("Nested parallel calls can deadlock with the python 2.7"
                        "multiprocessing backend.")
 
-    def nested_function_inner(i):
-        Parallel(n_jobs=2)(
-            delayed(exception_raiser)(j) for j in range(30))
-
-    def nested_function_outer(i):
-        Parallel(n_jobs=2)(
-            delayed(nested_function_inner)(j) for j in range(30))
-
     with raises(ValueError) as excinfo:
         Parallel(n_jobs=2, backend=backend)(
             delayed(nested_function_outer)(i) for i in range(30))
@@ -688,7 +690,7 @@ def test_parameterized_backend_context_manager(monkeypatch):
     assert _active_backend_type() == DefaultBackend
 
 
-def test_direct_parameterized_backend_context_manager():
+def test_directly_parameterized_backend_context_manager():
     assert _active_backend_type() == DefaultBackend
 
     # Check that it's possible to pass a backend instance directly,
@@ -708,23 +710,76 @@ def test_direct_parameterized_backend_context_manager():
     assert _active_backend_type() == DefaultBackend
 
 
+def sleep_and_return_pid():
+    sleep(.1)
+    return os.getpid()
+
+
+def get_nested_pids():
+    assert _active_backend_type() == ThreadingBackend
+    # Assert that the nested backend does not change the default number of
+    # jobs used in Parallel
+    assert Parallel()._effective_n_jobs() == 1
+
+    # Assert that the tasks are running only on one process
+    return Parallel(n_jobs=2)(delayed(sleep_and_return_pid)()
+                              for _ in range(2))
+
+
+
+class MyBackend(joblib._parallel_backends.LokyBackend):
+    """Backend to test backward compatibility with older backends"""
+    def get_nested_backend(self, ):
+        # Older backends only return a backend, without n_jobs indications.
+        return super(MyBackend, self).get_nested_backend()[0]
+
+
+register_parallel_backend('back_compat_backend', MyBackend)
+
+
 @with_multiprocessing
-def test_nested_backend_context_manager():
+@pytest.mark.parametrize('backend', ['threading', 'loky', 'multiprocessing',
+                                     'back_compat_backend'])
+def test_nested_backend_context_manager(backend):
     # Check that by default, nested parallel calls will always use the
     # ThreadingBackend
 
-    def get_nested_pids():
-        assert _active_backend_type() == ThreadingBackend
-        return Parallel(n_jobs=2)(delayed(os.getpid)() for _ in range(2))
+    with parallel_backend(backend):
+        pid_groups = Parallel(n_jobs=2)(
+            delayed(get_nested_pids)()
+            for _ in range(10)
+        )
+        for pid_group in pid_groups:
+            assert len(set(pid_group)) == 1
 
-    for backend in ['threading', 'loky', 'multiprocessing']:
-        with parallel_backend(backend):
-            pid_groups = Parallel(n_jobs=2)(
-                delayed(get_nested_pids)()
-                for _ in range(10)
-            )
-            for pid_group in pid_groups:
-                assert len(set(pid_group)) == 1
+
+@with_multiprocessing
+@pytest.mark.parametrize('n_jobs', [2, -1, None])
+@pytest.mark.parametrize('backend', PARALLEL_BACKENDS)
+def test_nested_backend_in_sequential(backend, n_jobs):
+    # Check that by default, nested parallel calls will always use the
+    # ThreadingBackend
+
+    def check_nested_backend(expected_backend_type, expected_n_job):
+        # Assert that the sequential backend at top level, does not change the
+        # backend for nested calls.
+        assert _active_backend_type() == BACKENDS[expected_backend_type]
+
+        # Assert that the nested backend in SequentialBackend does not change
+        # the default number of jobs used in Parallel
+        expected_n_job = effective_n_jobs(expected_n_job)
+        assert Parallel()._effective_n_jobs() == expected_n_job
+
+    Parallel(n_jobs=1)(
+        delayed(check_nested_backend)('loky', 1)
+        for _ in range(10)
+    )
+
+    with parallel_backend(backend, n_jobs=n_jobs):
+        Parallel(n_jobs=1)(
+            delayed(check_nested_backend)(backend, n_jobs)
+            for _ in range(10)
+        )
 
 
 @with_multiprocessing
@@ -877,12 +932,13 @@ print(Parallel(n_jobs=2, backend=backend)(
 
 
 @with_multiprocessing
-@parametrize('backend', PROCESS_BACKENDS +
-             ([] if sys.version_info[:2] < (3, 4) or mp is None
-              else ['spawn']))
+@parametrize('backend', PROCESS_BACKENDS)
 def test_parallel_with_interactively_defined_functions(backend):
     # When using the "-c" flag, interactive functions defined in __main__
     # should work with any backend.
+    if backend == "multiprocessing" and sys.platform == "win32":
+        pytest.skip("Require fork start method to use interactively defined "
+                    "functions with multiprocessing.")
     code = UNPICKLABLE_CALLABLE_SCRIPT_TEMPLATE_NO_MAIN.format(backend)
     check_subprocess_call(
         [sys.executable, '-c', code], timeout=10,
@@ -890,20 +946,26 @@ def test_parallel_with_interactively_defined_functions(backend):
 
 
 UNPICKLABLE_CALLABLE_SCRIPT_TEMPLATE_MAIN = """\
+import sys
+# Make sure that joblib is importable in the subprocess launching this
+# script. This is needed in case we run the tests from the joblib root
+# folder without having installed joblib
+sys.path.insert(0, {joblib_root_folder!r})
+
 from joblib import Parallel, delayed
 
 def run(f, x):
     return f(x)
 
-{}
+{define_func}
 
 if __name__ == "__main__":
-    backend = "{}"
+    backend = "{backend}"
     if backend == "spawn":
         from multiprocessing import get_context
         backend = get_context(backend)
 
-    callable_position = "{}"
+    callable_position = "{callable_position}"
     if callable_position == "delayed":
         print(Parallel(n_jobs=2, backend=backend)(
                 delayed(square)(i) for i in range(5)))
@@ -939,8 +1001,13 @@ square = lambda x: x ** 2
 @parametrize('callable_position', ['delayed', 'args', 'kwargs'])
 def test_parallel_with_unpicklable_functions_in_args(
         backend, define_func, callable_position, tmpdir):
+    if backend in ['multiprocessing', 'spawn'] and (
+            define_func != SQUARE_MAIN or sys.platform == "win32"):
+        pytest.skip("Not picklable with pickle")
     code = UNPICKLABLE_CALLABLE_SCRIPT_TEMPLATE_MAIN.format(
-        define_func, backend, callable_position)
+        define_func=define_func, backend=backend,
+        callable_position=callable_position,
+        joblib_root_folder=os.path.dirname(os.path.dirname(joblib.__file__)))
     code_file = tmpdir.join("unpicklable_func_script.py")
     code_file.write(code)
     check_subprocess_call(
@@ -1152,12 +1219,12 @@ def test_memmapping_leaks(backend, tmpdir):
     assert not os.listdir(tmpdir)
 
 
-def test_lambda_expression():
+@pytest.mark.parametrize('backend', [None, 'loky', 'threading'])
+def test_lambda_expression(backend):
     # cloudpickle is used to pickle delayed callables
-    for backend in ALL_VALID_BACKENDS:
-        results = Parallel(n_jobs=2, backend=backend)(
-            delayed(lambda x: x ** 2)(i) for i in range(10))
-        assert results == [i ** 2 for i in range(10)]
+    results = Parallel(n_jobs=2, backend=backend)(
+        delayed(lambda x: x ** 2)(i) for i in range(10))
+    assert results == [i ** 2 for i in range(10)]
 
 
 def test_delayed_check_pickle_deprecated():
@@ -1191,7 +1258,7 @@ def test_delayed_check_pickle_deprecated():
 
 
 @with_multiprocessing
-@parametrize('backend', ['multiprocessing', 'loky'])
+@parametrize('backend', PROCESS_BACKENDS)
 def test_backend_batch_statistics_reset(backend):
     """Test that a parallel backend correctly resets its batch statistics."""
     n_jobs = 2
@@ -1353,7 +1420,7 @@ def test_external_backends():
 def _recursive_backend_info(limit=3, **kwargs):
     """Perform nested parallel calls and introspect the backend on the way"""
 
-    with Parallel() as p:
+    with Parallel(n_jobs=2) as p:
         this_level = [(type(p._backend).__name__, p._backend.nesting_level)]
         if limit == 0:
             return this_level
@@ -1370,15 +1437,17 @@ def test_nested_parallelism_limit(backend):
 
     if cpu_count() == 1:
         second_level_backend_type = 'SequentialBackend'
+        max_level = 1
     else:
         second_level_backend_type = 'ThreadingBackend'
+        max_level = 2
 
     top_level_backend_type = backend.title() + 'Backend'
     expected_types_and_levels = [
         (top_level_backend_type, 0),
         (second_level_backend_type, 1),
-        ('SequentialBackend', 2),
-        ('SequentialBackend', 3)
+        ('SequentialBackend', max_level),
+        ('SequentialBackend', max_level)
     ]
     assert backend_types_and_levels == expected_types_and_levels
 
